# This file is generated by Lemonldap::NG::Manager::Build. Don't modify it by hand
package Lemonldap::NG::Handler::Lib::StatusConstants;

use strict;
use Exporter 'import';

our $VERSION = '2.23.1';

sub portalConsts {
    return {
          '-1' => 'PE_DONE',
          '-2' => 'PE_REDIRECT',
          '-3' => 'PE_INFO',
          '-4' => 'PE_SENDRESPONSE',
          '-5' => 'PE_IDPCHOICE',
          '-6' => 'PE_2FWAIT',
          '0' => 'PE_OK',
          '1' => 'PE_SESSIONEXPIRED',
          '10' => 'PE_BADCERTIFICATE',
          '100' => 'PE_PP_NOT_ALLOWED_CHARACTER',
          '101' => 'PE_PP_NOT_ALLOWED_CHARACTERS',
          '102' => 'PE_UPGRADESESSION',
          '103' => 'PE_NO_SECOND_FACTORS',
          '104' => 'PE_BAD_DEVOPS_FILE',
          '105' => 'PE_FILENOTFOUND',
          '106' => 'PE_OIDC_AUTH_ERROR',
          '107' => 'PE_UNKNOWNPARTNER',
          '108' => 'PE_UNAUTHORIZEDURL',
          '109' => 'PE_UNPROTECTEDURL',
          '110' => 'PE_RETRY_2FA',
          '111' => 'PE_PP_PASSWORD_TOO_LONG',
          '112' => 'PE_PP_PASSWORD_EXPIRES_SOON',
          '113' => 'PE_ISSUERTIMEOUT',
          '2' => 'PE_FORMEMPTY',
          '20' => 'PE_NO_PASSWORD_BE',
          '21' => 'PE_PP_ACCOUNT_LOCKED',
          '22' => 'PE_PP_PASSWORD_EXPIRED',
          '23' => 'PE_CERTIFICATEREQUIRED',
          '24' => 'PE_ERROR',
          '25' => 'PE_PP_CHANGE_AFTER_RESET',
          '26' => 'PE_PP_PASSWORD_MOD_NOT_ALLOWED',
          '27' => 'PE_PP_MUST_SUPPLY_OLD_PASSWORD',
          '28' => 'PE_PP_INSUFFICIENT_PASSWORD_QUALITY',
          '29' => 'PE_PP_PASSWORD_TOO_SHORT',
          '3' => 'PE_WRONGMANAGERACCOUNT',
          '30' => 'PE_PP_PASSWORD_TOO_YOUNG',
          '31' => 'PE_PP_PASSWORD_IN_HISTORY',
          '32' => 'PE_PP_GRACE',
          '33' => 'PE_PP_EXP_WARNING',
          '34' => 'PE_PASSWORD_MISMATCH',
          '35' => 'PE_PASSWORD_OK',
          '36' => 'PE_NOTIFICATION',
          '37' => 'PE_BADURL',
          '38' => 'PE_NOSCHEME',
          '39' => 'PE_BADOLDPASSWORD',
          '4' => 'PE_USERNOTFOUND',
          '40' => 'PE_MALFORMEDUSER',
          '41' => 'PE_SESSIONNOTGRANTED',
          '42' => 'PE_CONFIRM',
          '43' => 'PE_MAILFORMEMPTY',
          '44' => 'PE_BADMAILTOKEN',
          '45' => 'PE_MAILERROR',
          '46' => 'PE_MAILOK',
          '47' => 'PE_LOGOUT_OK',
          '48' => 'PE_SAML_ERROR',
          '49' => 'PE_SAML_LOAD_SERVICE_ERROR',
          '5' => 'PE_BADCREDENTIALS',
          '50' => 'PE_SAML_LOAD_IDP_ERROR',
          '51' => 'PE_SAML_SSO_ERROR',
          '52' => 'PE_SAML_UNKNOWN_ENTITY',
          '53' => 'PE_SAML_DESTINATION_ERROR',
          '54' => 'PE_SAML_CONDITIONS_ERROR',
          '55' => 'PE_SAML_IDPSSOINITIATED_NOTALLOWED',
          '56' => 'PE_SLO_ERROR',
          '57' => 'PE_SAML_SIGNATURE_ERROR',
          '58' => 'PE_SAML_ART_ERROR',
          '59' => 'PE_SAML_SESSION_ERROR',
          '6' => 'PE_LDAPCONNECTFAILED',
          '60' => 'PE_SAML_LOAD_SP_ERROR',
          '61' => 'PE_SAML_ATTR_ERROR',
          '62' => 'PE_OPENID_EMPTY',
          '63' => 'PE_OPENID_BADID',
          '64' => 'PE_MISSINGREQATTR',
          '65' => 'PE_BADPARTNER',
          '66' => 'PE_MAILCONFIRMATION_ALREADY_SENT',
          '67' => 'PE_PASSWORDFORMEMPTY',
          '68' => 'PE_CAS_SERVICE_NOT_ALLOWED',
          '69' => 'PE_MAILFIRSTACCESS',
          '7' => 'PE_LDAPERROR',
          '70' => 'PE_MAILNOTFOUND',
          '71' => 'PE_PASSWORDFIRSTACCESS',
          '72' => 'PE_MAILCONFIRMOK',
          '73' => 'PE_RADIUSCONNECTFAILED',
          '74' => 'PE_MUST_SUPPLY_OLD_PASSWORD',
          '75' => 'PE_FORBIDDENIP',
          '76' => 'PE_CAPTCHAERROR',
          '77' => 'PE_CAPTCHAEMPTY',
          '78' => 'PE_REGISTERFIRSTACCESS',
          '79' => 'PE_REGISTERFORMEMPTY',
          '8' => 'PE_APACHESESSIONERROR',
          '80' => 'PE_REGISTERALREADYEXISTS',
          '81' => 'PE_NOTOKEN',
          '82' => 'PE_TOKENEXPIRED',
          '83' => 'PE_WEBAUTHNFAILED',
          '84' => 'PE_UNAUTHORIZEDPARTNER',
          '85' => 'PE_RENEWSESSION',
          '86' => 'PE_WAIT',
          '87' => 'PE_MUSTAUTHN',
          '88' => 'PE_MUSTHAVEMAIL',
          '89' => 'PE_SAML_SERVICE_NOT_ALLOWED',
          '9' => 'PE_FIRSTACCESS',
          '90' => 'PE_OIDC_SERVICE_NOT_ALLOWED',
          '91' => 'PE_OID_SERVICE_NOT_ALLOWED',
          '92' => 'PE_GET_SERVICE_NOT_ALLOWED',
          '93' => 'PE_IMPERSONATION_SERVICE_NOT_ALLOWED',
          '94' => 'PE_ISSUERMISSINGREQATTR',
          '95' => 'PE_DECRYPTVALUE_SERVICE_NOT_ALLOWED',
          '96' => 'PE_BADOTP',
          '97' => 'PE_RESETCERTIFICATE_INVALID',
          '98' => 'PE_RESETCERTIFICATE_FORMEMPTY',
          '99' => 'PE_RESETCERTIFICATE_FIRSTACCESS'
        };

}

# EXPORTER PARAMETERS
our @EXPORT_OK = ( 'portalConsts' );
our %EXPORT_TAGS = ( 'all' => [ @EXPORT_OK, 'import' ], );

1;
